import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.Queue;

/**
 * Implements a breadth first search graph.
 * 
 * @author Alexander Weinhold
 * 
 * 
 *        This file belongs to the HLPPA-package - a java program that implements
 *        the highest label preflow-push algorithm. 
 * 
 *         This program is free software: you can redistribute it and/or modify
 *         it under the terms of the GNU General Public License as published by
 *         the Free Software Foundation, version 2 of the License.
 * 
 *         This program is distributed in the hope that it will be useful, but
 *         WITHOUT ANY WARRANTY; without even the implied warranty of
 *         MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 *         General Public License for more details.
 * 
 *         You should have received a copy of the GNU General Public License
 *         along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
public class BFSGraph {
	private BFSNode[] nodes;
	private int start;
	Queue<BFSNode> queue = new LinkedList<BFSNode>();

	public BFSGraph(String filename) {
		try {
			BufferedReader input = new BufferedReader(new FileReader(filename));
			String line;

			// Kommentare Wegspulen
			while ((line = input.readLine()) != null) {
				if (!line.trim().equals("") && !line.trim().substring(0, 1).equals("#")) {
					break;
				}
			}

			int n = Integer.parseInt(line.split("=")[1].trim());
			this.nodes = new BFSNode[n];

			// Anzahl der Kanten ist uninteressant
			input.readLine();
			// Quelle ebenfalls
			input.readLine();

			this.start = Integer.parseInt(input.readLine().split("=")[1].trim());

			int i = 0;
			while (i < this.nodes.length) {
				this.nodes[i] = new BFSNode();
				i++;
			}

			this.nodes[start].distance = 0;

			i = 0;
			while ((line = input.readLine()) != null) {
				if (line.trim().equals("")) {
					continue;
				}
				BFSNode.addEdges(i, nodes, line);
				i++;
			}

			input.close();

		} catch (IOException e) {
			e.printStackTrace();
			System.exit(-1);
		}

	}

	public int[] executeBFS() {
		int[] result = new int[nodes.length];

		queue.offer(nodes[start]);
		BFSNode node;
		while ((node = queue.poll()) != null) {

			for (BFSNode neighbor : node.adjacency) {

				if (neighbor.distance == -1) {

					queue.offer(neighbor);
					neighbor.distance = node.distance + 1;
				}
			}
		}

		for (int i = 0; i < nodes.length; i++) {
			result[i] = nodes[i].distance;
		}

		return result;
	}

	private static class BFSNode {
		ArrayList<BFSNode> adjacency;
		int distance;

		public BFSNode() {
			adjacency = new ArrayList<BFSNode>();
			distance = -1;
		}

		public static void addEdges(int target, BFSNode[] nodes, String line) {
			String[] adjacencyList = line.split(":");
			if (adjacencyList.length < 2)
				return;
			String[] edges = adjacencyList[1].trim().split(" ");
			for (String e : edges) {
				if (e.trim().equals(""))
					continue;
				String[] edge = e.split("c");
				int source = Integer.parseInt(edge[0].trim());
				nodes[source].adjacency.add(nodes[target]);
			}
		}

	}

}
